; ********************************************************************************************
; Macros fr SINAD entht macros fr FFT		
; ********************************************************************************************

; ACHTUNG: Beim XMEGA werden u.a. andere Namen fr SREG, SPH und SPL verwendet. Siehe hierzu:
;          im Pfad: c:\Programme\Atmel\Studio\7.0\packs\atmel\XMEGAA_DFP\1.1.68\avrasm\inc

.macro	pushsr
		push	temp		; Hilfsregister retten
		in	temp,cpu_sreg	; Statusregister		ACHTUNG: XMega-Name!!!
		push	temp		; ... retten
.endm
; --------------------------------------
.macro	popsr
		pop	temp		; Statusregister
		out	cpu_sreg,temp	; ...zurckladen		ACHTUNG: XMega-Name!!!
		pop	temp		; Hilfsregister zurck laden
.endm
; --------------------------------------
.macro	stsw
		sts	(@0),@1L	; speichere Registerpaar @1
		sts	(@0)+1,@1H	; ...nach (@0)
.endm
; --------------------------------------
.macro	cpwi				; 16Bit-Register mit Konstante vergleichen
		cpi	@0H,high(@1)	
		brcs	exit		; Konstante > Wortregister
		brne	exit		; Konstante < Wortregister
		cpi	@0L,low(@1)	; Ergebnis in SREG
EXIT:
.endm
; --------------------------------------
.macro	incram
		lds	temp,@0		; Byte aus RAM laden
		inc	temp		; ...erhhen
		sts	@0,temp		; ...und zurck speichern
.endm
; --------------------------------------
.macro	decram
		lds	temp,@0		; Byte aus RAM laden
		tst	temp		; Zhler schon abgelaufen?
		breq	exit		; ja, nichts ndern
		dec	temp		; Zhler vermindern
		sts	@0,temp		; ...und zurck speichern
EXIT:
.endm
; --------------------------------------
.macro	brls
		breq	@0		; gleich --> Sprung
		brcs	@0		; kleiner --> Sprung
EXIT:
.endm
; --------------------------------------


; ********************************************************************************************
; ********************************************************************************************
;
; Macros ab hier: Copyright-Vermerk:
;
;	FFT-Routinenen: (c) ChaN, 2004; http://elm-chan.org/
;	nderungen sind im Kommentar mit "DC7GB:" gekennzeichnet.
;
; ********************************************************************************************
; ********************************************************************************************

; -----------------------------------------------
; 16Bit x 16Bit = 32Bit: @1 x @0 = (@3 ; @2)
; (F8.8 x F8.8)*2 signed

.macro	FMULS16				; 20clk
.ifndef	_ORIGINAL
		clr	r15		; DC7GB:	Ergnzung, um globale Definition zu vermeiden
.endif
		fmuls	@0H, @1H
		movw	@2L, T0L
		fmul	@0L, @1L
		movw	@3L, T0L
		adc	@2L, r15
		fmulsu	@0H, @1L
		sbc	@2H, r15
		add	@3H, T0L
		adc	@2L, T0H
		adc	@2H, r15
		fmulsu	@1H, @0L
		sbc	@2H, r15
		add	@3H, T0L
		adc	@2L, T0H
		adc	@2H,r15
.endmacro


.macro	SQRT32				; 526..542clk
		clr	T6L
		clr	T6H
		clr	T8L
		clr	T8H
		ldi	AL, 1
		ldi	AH, 0
		clr	BL
		clr	BH
		ldi	CL, 16		; 16 Iterationen
;
SQRT16_34:	lsl	T2L
		rol	T2H
		rol	T4L
		rol	T4H
		rol	T6L
		rol	T6H
		rol	T8L
		rol	T8H
;
		lsl	T2L
		rol	T2H
		rol	T4L
		rol	T4H
		rol	T6L
		rol	T6H
		rol	T8L
		rol	T8H
		brpl	sqrt16_6	; PC+6
		add	T6L, AL
		adc	T6H, AH
		adc	T8L, BL
		adc	T8H, BH
		rjmp	sqrt16_5	; PC+5
;
SQRT16_6:	sub	T6L, AL
		sbc	T6H, AH
		sbc	T8L, BL
		sbc	T8H, BH
SQRT16_5:	lsl	AL
		rol	AH
		rol	BL
		andi	AL, 0b11111000
		ori	AL, 0b00000101
		sbrc	T8H, 7
		subi	AL, 2
		dec	CL		; Iterationen fertig?
		brne	sqrt16_34	; nein, PC-34
;
		lsr	BL
		ror	AH
		ror	AL
		lsr	BL
		ror	AH
		ror	AL
.endm

;------------------------------------------------;
; Load/store word from/to direct memory/immediate

.macro		ldiw
		ldi	@0L,low(@1)
		ldi	@0H,high(@1)
.endm

.macro		ldsw
		lds	@0L,@1
		lds	@0H,@1+1
.endm

.macro		lddw
		ldd	@0L,@1
		ldd	@0H,@1+1
.endm

.macro		ldw
		ld	@0L,@1
		ld	@0H,@1
.endm

.macro		stw
		st	@0,@1L
		st	@0,@1H
.endm


;------------------------------------------------;
; Store immediate into indirect memory via r16

.macro		sti
		ldi	r16,@1
		st	@0,r16
.endm

.macro		stdi
		ldi	r16,@1
		std	@0,r16
.endm

.macro		muli
		ldi	r16,@1
		mul	@0,r16
.endm


;------------------------------------------------;
; add/sub/subc/cp/cpc/lsl/lsr/rol/ror to register pair
;

.macro		addiw
		subi	@0L,low(-(@1))
		sbci	@0H,high(-(@1))
.endm

.macro		subiw
		subi	@0L,low(@1)
		sbci	@0H,high(@1)
.endm

.macro		addw
		add	@0L,@1L
		adc	@0H,@1H
.endm

.macro		adcw
		adc	@0L,@1L
		adc	@0H,@1H
.endm

.macro		subw
		sub	@0L,@1L
		sbc	@0H,@1H
.endm

.macro		sbcw
		sbc	@0L,@1L
		sbc	@0H,@1H
.endm

.macro		cpiw
		cpi	@0L,low(@1)
		ldi	r16,high(@1) 
		cpc	@0H,r16
.endm

.macro		lpmw
		lpm	@0L, @1
		lpm	@0H, @1
.endm


;------------------------------------------------;
; Add immediate to register

.macro		addi
		subi	@0,-(@1)
.endm


;------------------------------------------------;
; Long branch


.macro		rjne
		breq	PC+2
		rjmp	@0
.endm

.macro		rjeq
		brne	PC+2
		rjmp	@0
.endm

.macro		rjcc
		brcs	PC+2
		rjmp	@0
.endm

.macro		rjcs
		brcc	PC+2
		rjmp	@0
.endm

.macro		rjtc
		brts	PC+2
		rjmp	@0
.endm

.macro		rjts
		brtc	PC+2
		rjmp	@0
.endm

.macro		retcc
		brcs	PC+2
		ret
.endm

.macro		retcs
		brcc	PC+2
		ret
.endm

.macro		reteq
		brne	PC+2
		ret
.endm

.macro		retne
		breq	PC+2
		ret
.endm



