.list
; ********************************************************************************************
; SINAD-Messgerät V2.1 mit ATxmega32A4 @ 32 MHz
; ********************************************************************************************

.include	"macro_fft.inc"

; ---------------------------------------------------------------------------------------------
;	Copyright-Vermerk:
;
;	Alle Routinen und Include-Dateien, sofern nicht anders angegeben, lizenziert unter:
;	Creative Commons BY-NC-SA 4.0
;
;	FFT-Routinenen: (c) ChaN, 2004; http://elm-chan.org/
;	DISP_-Routinen: (c) dc7gb@vfdb.org, 2012-2020; https://creativecommons.org/licenses/by-nc-sa/4.0/
;
;.equ	_ORIGINAL = 1	; verwendet die Original sqrt-Kompression von ChaN
;
; ---------------------------------------------------------------------------------------------
;	Brown-out	: 2,9V		(continuously)
;	FUSEBYTE4JTAGEN	: off		(Haken entfernen)	FUSEBYTE4 = $FF
; ---------------------------------------------------------------------------------------------
;	ACHTUNG: Beim XMEGA werden u.a. andere Namen für SREG, SPH und SPL verwendet. Siehe hierzu:
;		 im Pfad: c:\Programme\Atmel\Studio\7.0\packs\atmel\XMEGAA_DFP\1.1.68\avrasm\inc
; ---------------------------------------------------------------------------------------------

; ----------------------------------------------
; Port_A = VPort_0:

.equ	CDDRA		= 0b00110000		; Datenrichtung A
;
.equ	PA_0		= BIT0			; (40) 
.equ	PA_1		= BIT1			; (41) 
.equ	ADC2_P		= BIT2			; (42) ANA: (+) Eingang
.equ	ADC3_M		= BIT3			; (43) ANA: (-) Eingang
.equ	POWERONHOLD	= BIT4			; (44) OUT: Einschalt-Haltefunktion
.equ	TEST		= BIT5			; ( 1) OUT: Testausgang
.equ	PA_6		= BIT6			; ( 2)
.equ	PA_7		= BIT7			; ( 3)

; ----------------------------------------------
; Port_B = VPort_1:

.equ	CDDRB		= 0b00000000		; Datenrichtung B
;
.equ	PB_0		= BIT0			; ( 4) 
.equ	J1		= BIT1			; ( 5) INP: Lötbrücke J1
.equ	DAC_0		= BIT2			; ( 6) ANA: Sinus-Ausgang DAC0
.equ	PB_3		= BIT3			; ( 7) 

; ----------------------------------------------
; Port_C = VPort_2:

.equ	CDDRC		= 0b00111100		; Daternrichtung C
;
.equ	PC_0		= BIT0			; (10) 
.equ	PC_1		= BIT1			; (11) 
.equ	OVERFLOW	= BIT2			; (12) OUT: Error/Overlow-LED
.equ	EPOT_CLK	= BIT3			; (13) OUT: E-Poti-Takt
.equ	EPOT_NCS	= BIT4			; (14) OUT: E-Poti-NCS
.equ	EPOT_SDI	= BIT5			; (15) OUT: E-Poti-SDI
.equ	PC_6		= BIT6			; (16)
.equ	PC_7		= BIT7			; (17)

; ----------------------------------------------
; Port_D = VPort_3:

.equ	CDDRD		= 0b11111000		; Datenrichtung D
;
.equ	DG_A		= BIT0			; (20) INP: Drehgeber-A (QDPH0)
.equ	DG_B		= BIT1			; (21) INP: Drehgeber-B (QDPH90)
.equ	DG_E		= Bit2			; (22) INP: Drehgeber-E
.equ	LCD_SI		= BIT3			; (23) OUT: SI-Display
.equ	LCD_SCL		= BIT4			; (24) OUT: SCL-Display
.equ	LCD_A0		= BIT5			; (25) OUT: A0-Display
.equ	LCD_RST		= BIT6			; (26) OUT: RST-Display
.equ	LCD_CS		= BIT7			; (27) OUT: CS-Display

; ----------------------------------------------
; Port_E:

.equ	CDDRE		= 0b00001000		; Datenrichtung E
;
.equ	PE_0		= BIT0			; (28) 
.equ	PE_1		= BIT1			; (29)
.equ	PE_2		= BIT2			; (32)
.equ	TP_TAKT		= BIT3			; (33) OUT: COMPARE_D (400 kHz)

; ----------------------------------------------
; Systemflags 1

.equ	flags1		= GPIO_GPIOR1
.equ	TIM10MS		= 0			; 1= 10ms-Timer ist abgelaufen
.equ	TIM100MS	= 1			; 1= 100ms-Timer ist abgelaufen
.equ	TIM1000MS	= 2			; 1= 10000ms-Timer ist abgelaufen
.equ	ADCBUFRDY	= 3			; 1= neuer ADC-Buffer zur Auswertung vorhanden
.equ	CAPTBUFWR	= 4			; 0= ADC schreibt in CAPTBUF0 / 1= ADC schreibt in CAPTBUF1
.equ	F1_5		= 5			; 
.equ	F1_6		= 6			; 
.equ	F1_7		= 7			; 

; ----------------------------------------------
; Systemflags 2

.equ	flags2		= GPIO_GPIOR2
.equ	TASTEDRUCK	= 0			; 1= Taste war beim letzten Durchlauf gedrückt
.equ	TASTEENTPRELLT	= 1			; 1= Taste gedrückt und entprellt
.equ	TASTEWAIT	= 2			; 1= Tasten-Langzeitauswertung abgeschlossen 
.equ	TASTEKURZ	= 3			; 1= Tastendruck kurz erkannt
.equ	TASTELANG	= 4			; 1= Tastendruck lang erkannt
.equ	F2_5		= 5			; 
.equ	F2_6		= 6			; 
.equ	F2_7		= 7			; 

; --------------------------------------
; Zeiten und Zähler

.equ	QUARZ		= 32000000			; Quarzfrequenz in Hz
.equ	SYSTAKT		= 20000				; 50µs I/O-Rate
.equ	ADCTAKT		= 8000				; 8kHz Abtasttakt
.equ	FILTERTAKT0	= 400000			; für MAX7407 (volle Bandbreite)
.equ	FILTERTAKT1	= 343800			; ...Sprach-Filter
.equ	FILTERTAKT2	= 131300			; ...CW-Filter
.equ	TICS_C0		= (QUARZ/1)/SYSTAKT-1		; 1599 mit Vorteiler 1
.equ	TICS_C1		= (QUARZ/1)/ADCTAKT-1		; 3999 mit Vorteiler 1
.equ	TICS_E00	= (QUARZ/1)/FILTERTAKT0-1	; 79 mit Vorteiler 1 (400000 Hz)
.equ	TICS_E01	= (QUARZ/1)/FILTERTAKT1-1	; 92 mit Vorteiler 1 (347826 Hz)
.equ	TICS_E02	= (QUARZ/1)/FILTERTAKT2-1	; 242 mit Vorteiler 1 (131687 Hz)

; ----------------------------------------------
; Buffer-Längen:

.equ	FFT_N		= 128			; Abtastlänge in 16Bit-Worten

; ----------------------------------------------
; Bit-Codes

.equ	BIT0		= 0
.equ	BIT1		= 1
.equ	BIT2		= 2
.equ	BIT3		= 3
.equ	BIT4		= 4
.equ	BIT5		= 5
.equ	BIT6		= 6
.equ	BIT7		= 7

;---------------------------------------
; Verzögerung in 10ms-Schritten:

.equ	DELAY100MS	= 10			; DG-Entprellung
.equ	DELAY500MS	= 50			; DG-Langdruck-Auswertung
.equ	DELAY1S		= 100			; Overflow Haltezeit für SINAD-Wert

;---------------------------------------
; Verzögerung in 100ms-Schritten:

.equ	DLY3S		= 30			; Display-Anzeigezeit, PowerOff

; ----------------------------------------------
; ASCII-Codes:

.equ	CR		= $D			; Zurück zu dispx
.equ	LF		= $A			; neue Zeile (dispy + Fonthöhe)
.equ	SPACE		= $20			; Zwischenraum (dispx + Fontbreite)

; ----------------------------------------------
; sonstige Konstanten:

.equ	POWERONDELAY	= 81			; ergibt bei 32 MHz ca. 500ms Delay
.equ	LOWSCHWELLE	= 86			; empirisch ermittelte Schwelle

; ----------------------------------------------
; Registerdefinitionen 1: (c) http://elm-chan.org/

.def	T0L		= r0
.def	T0H		= r1
.def	T2L		= r2
.def	T2H		= r3
.def	T4L		= r4
.def	T4H		= r5
.def	T6L		= r6
.def	T6H		= r7
.def	T8L		= r8
.def	T8H		= r9
.def	T10L		= r10
.def	T10H		= r11
.def	T12L		= r12
.def	T12H		= r13
.def	T14L		= r14
.def	T14H		= r15
;-------------------------------
.def	AL		= r16
.def	AH		= r17
.def	BL		= r18
.def	BH		= r19
.def	CL		= r20
.def	CH		= r21
.def	DL		= r22
.def	DH		= r23
.def	EL		= r24
.def	EH		= r25


; ----------------------------------------------
; Registerdefinitionen 2:

.def	akku0		= r0
.def	akku1		= r1
.def	akku2		= r2
.def	akku3		= r3
.def	akku4		= r4
.def	akku5		= r5
.def	akku6		= r6
.def	akku7		= r7
.def	akku8		= r8
;
.def	rest		= r14		; 8Bit Divisionsrest
.def	const		= r15		; Konstantenregister

;------ ACHTUNG: Mit r16..31 sind folgende Registerbefehle möglich ------------------------------- 
;	adiw, cbr, cpi, sbr, sbci, sbiw, subi, ldi, andi, ori

.def	temp		= r16		; Arbeitsregister
.def	buffer		= r17		; Hilfsregister
.def	counter		= r18		; Zähler für Schleifen (Bitzähler)
.def	index		= r19		; Zeigerregister



.dseg
; ***********************************************************************************************
; RAM-Bereich:
; ***********************************************************************************************

memstart:					; weitere Definitionen in disp.asm und data.asm

.cseg
; -----------------------------------------------------------------------------------------------
; Program code area
; -----------------------------------------------------------------------------------------------
.org	$00

; ----------------------------------------------
		jmp	START			; System-Einsprungpunkt
.org	$02
OSCF_INT:	jmp	OSCF_INT		; Crystal oscillator failure interrupt vector (NMI)
.org	$04
PORTC_INT0_v:	jmp	PORTC_INT0_v		; Port C interrupt base
.org	$08
PORTR_INT:	jmp	PORTR_INT		; Port R interrupt base
.org	$0c
DMA_INT:	jmp	DMA_INT			; DMA controller interrupt base
.org	$14
RTC_INT:	jmp	RTC_INT			; Real time counter interrupt base
.org	$18
TWIC_TWIS_v:	jmp	TWIC_TWIS_v		; Two-Wire Interface on Port C interrupt base_-
.org	$1c
		jmp	TCC0_OVF_v		; TCC0 Overflow Interrupt
TCC0_ERR_v:	jmp	TCC0_ERR_v		; TCC0 Error Interrupt
TCC0_CCA_v:	jmp	TCC0_CCA_v		; TCC0 Compare or Capture A Interrupt
TCC0_CCB_v:	jmp	TCC0_CCB_v		; TCC0 Compare or Capture B Interrupt
TCC0_CCC_v:	jmp	TCC0_CCC_v		; TCC0 Compare or Capture C Interrupt
TCC0_CCD_v:	jmp	TCC0_CCD_v		; TCC0 Compare or Capture D Interrupt
.org	$28
TCC1_OVF_v:	jmp	TCC1_OVF_v		; TCC1 Overflow Interrupt
TCC1_ERR_v:	jmp	TCC1_ERR_v		; TCC1 Error Interrupt
TCC1_CCA_v:	jmp	TCC1_CCA_v		; TCC1 Compare or Capture A Interrupt
TCC1_CCB_v:	jmp	TCC1_CCB_v		; TCC1 Compare or Capture B Interrupt
.org	$30
SPIC_INT:	jmp	SPIC_INT		; SPI on port C interrupt vector
.org	$32
USARTC0_RXC_v:	jmp	USARTC0_RXC_v		; USART 0 on port C Reception Complete Interrupt
USARTC0_DRE_v:	jmp	USARTC0_DRE_v		; USART 0 on port C Data Register Empty Interrupt
USARTC0_TXC_v:	jmp	USARTC0_TXC_v		; USART 0 on port C Transmission Complete Interrupt
.org	$38
USARTC1_INT:	jmp	USARTC1_INT		; USART 1 on port C interrupt base
.org	$3e
AES_INT:	jmp	AES_INT			; AES interrupt vector
.org	$40
NVM_INT:	jmp	NVM_INT			; Nonvolatile Memory interrupt base
.org	$44
PORTB_INT:	jmp	PORTB_INT		; Port B interrupt base
.org	$56
PORTE_INT:	jmp	PORTE_INT		; Port E interrupt base
.org	$5a
TWIE_INT_v:	jmp	TWIE_INT_v		; Two-wire Interface on Port E interrupt base
.org	$5e
TCE0_INT:	jmp	TCE0_INT		; Timer/counter 0 on port E interrupt base
.org	$6a
TCE1_INT:	jmp	TCE1_INT		; Timer/counter 1 on port E interrupt base
.org	$74
USARTE0_INT:	jmp	USARTE0_INT		; USART 0 on port E interrupt base
.org	$80
PORTD_INT0_v:	jmp	PORTD_INT0_v		; Port D interrupt base
.org	$84
PORTA_INT0_v:	jmp	PORTA_INT0_v		; Port A interrupt base
.org	$88
ACA_INT:	jmp	ACA_INT			; Analog Comparator on Port A interrupt base
.org	$8e
		jmp	ADCA_INT		; Analog to Digital Converter on Port A interrupt base
.org	$9a
TCD0_INT:	jmp	TCD0_INT		; Timer/counter 0 on port D interrupt base
.org	$a6
TCD1_INT:	jmp	TCD1_INT		; Timer/counter 1 on port D interrupt base
.org	$ae
SPID_INT:	jmp	SPID_INT		; SPI on port D interrupt vector
.org	$b0
USARTD0_INT:	jmp	USARTD0_INT		; USART 0 on port D interrupt base
.org	$b6
USARTD1_INT:	jmp	USARTD1_INT		; USART 1 on port D interrupt base
.org	$fa
USB_INT:	jmp	USB_INT			; USB on port D interrupt base


;-----------------------------------------------
; Programmstart:

.org	$100					;; Versionshinweis siehe: data.asm				

;-----------------------------------------------
; System starten:

START:

; ----------------------------------------------
; Externen Oszillator mít 2x PLL verwenden: 

		ldi	temp,OSC_XOSCEN_bm	;; externen Oszillator
		sts	OSC_CTRL,temp		;; ...einstellen
		ldi	temp,0b00000000		;; A=1, B=1, C=1
		sts	CLK_PSCTRL,temp		;; ...einstellen
OSCWAIT0:	lds	temp,OSC_STATUS
		sbrs	temp,OSC_XOSCRDY_bp	;; ready?
		rjmp	oscwait0		;; nein...
		ldi	temp,$d8		;; Signature Protected IO-Register
		sts	CPU_CCP,temp		;; Freigabe für 4 CLK-Zyklen aktivieren
		ldi	temp,CLK_SCLKSEL_XOSC_gc
		sts	CLK_CTRL,temp
;
		ldi	temp,(OSC_PLLSRC_gm | 2);; PLL-Multiplikation auf 2
		sts	OSC_PLLCTRL,temp	;; ...einstellen --> CLK=32 MHz
		ldi	temp,OSC_PLLEN_bm	;; PLL
		sts	OSC_CTRL,temp		;; ...einstellen
OSCWAIT1:	lds	temp,OSC_STATUS
		sbrs	temp,OSC_PLLRDY_bp	;; ready?
		rjmp	oscwait1		;; nein...
		ldi	temp,$d8		;; Signature Protected IO-Register
		sts	CPU_CCP,temp		;; Freigabe für 4 CLK-Zyklen aktivieren
		ldi	temp,CLK_SCLKSEL_PLL_gc
		sts	CLK_CTRL,temp

; ---------------------------------------------
; Speicher löschen:

		clr	temp
		ldiw	Z,memstart
INIT1:		st	Z+,temp			;; RAM-Zellen ab <memstart> löschen
		cpwi	Z,(memende+1)		;; RAM-Ende erreicht?
		brne	init1			;; nein...

; ---------------------------------------------
; Virtual Port 0..3 einstellen:

		ldi	temp,0b00010000		;; Port-B und Port-A wird
		sts	PORTCFG_VPCTRLA,temp	;; ...Virtual Port-1 und Port-0
		ldi	temp,0b00110010		;; Port-D und Port-C wird
		sts	PORTCFG_VPCTRLB,temp	;; ...Virtual Port-3 und Port-2

; ---------------------------------------------
; I/O-Richtung einstellen:

		ldi	temp,CDDRA		;; PortA
		sts	PORTA_DIRSET,temp	;; ...einstellen
;
		ldi	temp,CDDRB		;; PortB
		sts	PORTB_DIRSET,temp	;; ...einstellen
		ldi	temp,0b00011000		;; Totem-Pole
		sts	PORTB_PIN1CTRL,temp	;; ...an PB1=J1 einstellen
;
		ldi	temp,CDDRC		;; PortC
		sts	PORTC_DIRSET,temp	;; ...einstellen
;
		ldi	temp,CDDRD		;; PortD
		sts	PORTD_DIRSET,temp	;; ...einstellen
		ldi	temp,0b01011000		;; Inv-Inp --> Drehrichtung: (rechts = mehr)
		sts	PORTD_PIN0CTRL,temp	;; ...Pull-Up an PD2=DG-A 
		ldi	temp,0b00011000		;; Pull-Up
		sts	PORTD_PIN1CTRL,temp	;; ...an PD1=DG-B
		sts	PORTD_PIN2CTRL,temp	;; ...an PD0=DG-E
;
		ldi	temp,CDDRE		;; PortE
		sts	PORTE_DIRSET,temp	;; ...einstellen
		
; ---------------------------------------------
; PowerOn-Hold ausgeben:


PON_WAIT:	call	test_toggle		;; 100ms-Pulse zur Fehlersuche
		sbis	VPORT3_IN,DG_E		;; DG immer noch gedrückt?
		rjmp	pon_wait		;; ja...
		sbi	VPORT0_OUT,POWERONHOLD	;; Spannung halten

; ---------------------------------------------
; Timer initialisieren:

; Timer-C0 für 50us (Systemtimer):

TIMER_C0:	ldi	temp,0b0001		;; CLK-Teiler 1
		sts	TCC0_CTRLA,temp		;; ...einstellen
		ldi	temp,0			;; kein COMPARE, WGM-Normal-Mode
		sts	TCC0_CTRLB,temp		;; ...einstellen
		ldiw	Z,TICS_C0		;; Periode
		stsw	TCC0_PERBUF,Z		;; ...einstellen
		ldi	temp,0b10		;; IRQ-Level für Channel-A
		sts	TCC0_INTCTRLA,temp	;; ...auf "medium" stellen, 

; Timer für ADC-Start

TIMER_C1:	ldi	temp,0b0001		;; CLK-Teiler 1
		sts	TCC1_CTRLA,temp		;; ...einstellen
		ldi	temp,0			;; kein COMPARE, WGM-Normal-Mode
		sts	TCC1_CTRLB,temp		;; ...einstellen
		ldiw	Z,(TICS_C1/2)		;; Compare-A (Phasenlage) für ADC_CH0
		stsw	TCC1_CCABUF,Z		;; ...(Event Channel 7) einstellen
		ldiw	Z,TICS_C1		;; Periode
		stsw	TCC1_PERBUF,Z		;; ...einstellen

; Zähler für Quadratur-Dekoder:

TIMER_D0:	ldi	temp,0b1		;; Clock :1
		sts	TCD1_CTRLA,temp		;; ...aktivieren
		ldi	temp,0b01101100		;; QDEC, Event-Channel 4
		sts	TCD1_CTRLD,temp		;; ...selektieren
;
		ldiw	Z,399			;; Periode (4 * MAX) -1
		stsw	TCD0_PERBUF,Z		;; ...einstellen

; Timer für Aliasing-Filter MAX7407 (400kHz)

TIMER_E0:	ldi	temp,0b0001		;; CLK-Teiler 1
		sts	TCE0_CTRLA,temp		;; ...einstellen
		ldi	temp,0b10000011		;; Compare-D aktiv, Single-Slope-Mode
		sts	TCE0_CTRLB,temp		;; ...einstellen (OC0D: PF3 = Pin49)
		ldiw	Z,TICS_E00		;; Timer für externen Takt
		rcall	set_timer_e0		; ...des MAX7407 einstellen

; ----------------------------------------------
; Event System:

		ldi	temp,0b11001100		;; TCC1-Compare-A
		sts	EVSYS_CH7MUX,temp	;; ...als Event-Channel 7 einstellen
;
		ldi	temp,0b01101000		;; QDPH0 (PD0)
		sts	EVSYS_CH4MUX,temp	;; ...als Event-Channel 4 einstellen
		ldi	temp,0b00001111		;; Quadratur enable, 8 Takte filtern
		sts	EVSYS_CH4CTRL,temp	;; ...für Event-Channel 4 einstellen

; ---------------------------------------------
; ADC an PA3 (ADC3-, Pin43) und PA2 (ADC2+, Pin42):

ADCA_CH0:	ldi	temp,0b00000001		;; ADC_A0
		sts	ADCA_CTRLA,temp		;; ...frei geben
		ldi	temp,0b11010000		;; Low-Imp, Low Current, signed Mode, 12Bit-right
		sts	ADCA_CTRLB,temp		;; ...einstellen
		ldi	temp,0b00000010		;; Ref=1V, Bandgap enable
		sts	ADCA_REFCTRL,temp	;; ...aktivieren

		ldi	temp,0b00111001		;; verwende Event Channel 7 (TCC1-Compare-A)
		sts	ADCA_EVCTRL,temp	;; ...zum Start der Abtastung
		ldi	temp,0b00000100		;; SYSTAKT/64 = 0,5M Samples/s
		sts	ADCA_PRESCALER,temp	;; ...einstellen
		ldi	temp,0b00000010		;; Gain=1, DIFF-Mode
		sts	ADCA_CH0_CTRL,temp	;; ...setzen
		ldi	temp,0b00010011		;; ADC2:+, ADC3:-
		sts	ADCA_CH0_MUXCTRL,temp	;; ...einstellen
		ldi	temp,0b0010		;; Conversion Complete, Medium Level
		sts	ADCA_CH0_INTCTRL,temp	;; ...einstellen
			
; ---------------------------------------------
; DAC an PB2 (DAC0, Pin6):

		ldi	temp,0b00000111		;; DAC0 verwenden, Low-Power-Mode 
		sts	DACB_CTRLA,temp		;; ...DACs aktivieren
		ldi	temp,0b00000000		;; Single-Channel DAC0, kein Auto-Trigger
		sts	DACB_CTRLB,temp		;; ...selektieren
		ldi	temp,0b00000000		;; Referenz: Intern 1V, 12Bit right-adjusted
		sts	DACB_CTRLC,temp		;; ...einstellen
		ldi	temp,0b00000000		;; keine Ereignis
		sts	DACB_EVCTRL,temp	;; ...Steuerung

; ---------------------------------------------
; Interrupt-Controller frei geben:

		clr	temp			;; Priority-Level
		sts	PMIC_INTPRI,temp	;; ...auf 0 setzen
		ldi	temp,0b0111		;; H-, M- und L-Interrupts
		sts	PMIC_CTRL,temp		;; ...frei geben

; ---------------------------------------------
; Variablen initialisieren:

		ldiw	Z,(2*SINTAB12)		;; Zeiger der Sinus-Tabelle
		stsw	sinusrp,Z		;; ...auf Anfang stellen
		ldiw	Z,CAPTBUF0		;; initialisiere
		stsw	captbufp,Z		;; ...ADC-Schreibpointer
		cbi	flags1,CAPTBUFWR	;; ADC verwendet CAPTBUF0
		ldi	temp,200		;; 10ms-Timer
		sts	timer_10ms,temp		;; ...initialisieren
		ldi	temp,10			;; 1000ms-Intervall
		sts	timer_1000ms,temp	;; ...initialisieren
		ldi	temp,1			;; NF-Filterauswahl
		sts	bbsel,temp		;; ...auf SPRACHE
		call	set_filter		;; ...setzen
		clr	temp			;; Verstärkung auf .0dB setzen
		call	p0_set			;; ...und Poti einstellen
		sts	dgstate,temp		;; DG-EInstellung auf vein setzen
		ldi	temp,-1			;; ...und beim ersten Aufruf
		sts	dgstatelast,temp	;; ...Zähler einstellen
		call	disp_init		;; DISP-System initialisieren
		call	disp_clrram		;; Display-RAM löschen
		ldi	temp,DELAY100MS		;; Overflow-LED 
		sts	timer_overflow,temp	;; ...aktivieren

; ---------------------------------------------
; Sleep-Mode auf Idle setzen

		ldi	temp,0b1		;; SMODE: Idle=000 / SEN=1:
		sts	SLEEP_CTRL,temp		;; ...setzen

; ********************************************************************************************
; Hauptprogramm:
; ********************************************************************************************

MAIN:		clr	temp			;; beginne mit
		sts	state,temp		;; ...Startbildschirm
		ldi	temp,DLY3S		;; Anzeigezeit
		sts	timer_wait,temp		;; ...einstellen

; Watchdog aktivieren:

		ldi	buffer,0b001011		;; 32ms Watchdog-Timer
		ldi	temp,$d8		;; Signature Protected IO-Register
		sts	CPU_CCP,temp		;; Freigabe für 4 CLK-Zyklen aktivieren
		sts	WDT_CTRL,buffer		;; Watchdog starten

; ----------------------------------------------
; Hauptschleife:

WAIT:		sei				; *** Alle Interrupts freigeben
		wdr				; periodischer Watchdog-RESET
		sbis	flags1,ADCBUFRDY	; neuer ADC-Buffer vorhanden?
		rjmp	wait			; nein...
		cbi	flags1,ADCBUFRDY	; nächsten 16ms-Timerablauf ermöglichen

; nächstes Main-Programm auswählen:

		ldiw	Z,(2*JUMP)		; Sprungtabelle der Main-Programme
		lds	temp,state		; aufzurufendes Main-Programm
		rcall	select			; ...starten

; bei jedem Aufruf ausführen:

		rcall	dg_manager		; Drehgeber auswerten
		rcall	errorled		; LED ein bei Overflow
		sbic	flags1,TIM100MS		; Display auffrischen?
		call	disp_refresh		; ja, Daten ausgeben, RAM löschen	[3367,38 µs, 107756 cyc]
		cbi	flags1,TIM100MS		; nur 10x pro Sekunde auffrischen
;
		rjmp	wait


; ----------------------------------------------
; Sprungtabelle für Main-Programme 

JUMP:		.dw	Z0			; 0: Startbildschirm
		.dw	Z1			; 1: Ausschaltbildschirm
		.dw	Z2			; 2: Diagrammanzeige
		.dw	Z3			; 3: Menüauswahl



; ********************************************************************************************
; Main-Programme:
; ********************************************************************************************


; ----------------------------------------------
; Startbildschirm aufbauen:

Z0:		lds	temp,timer_wait		; Anzeigezeit
		tst	temp			; ...vorbei?
		brne	z0_display		; nein...
		ldi	temp,2			; rufe nächsten
		sts	state,temp		; ...Zustand auf
Z0_DISPLAY:	ldi	temp,13			; Textanfang X
		sts	dispx,temp		; ...einstellen
		ldi	temp,20			; Position vom oberen Rand
		sts	dispy,temp		; ...einstellen
		ldiw	Z,(2*STARTMSG)		; Startmeldung
		ldi	counter,-1		; ...ohne Ausgabebegrenzung
		call	disp_printrom		; ...ausgeben
;
		call	rahmen			; Rahmen zeichnen	
; -------------
		ret

	
; ----------------------------------------------
; Endbildschirm aufbauen:

Z1:		call	disp_setram		; Bildschirm löschen
;
		call	disp_font2		; großen Font einstellen
		ldi	temp,57			; X-Mitte
		sts	dispx,temp		; ...setzen
		ldi	temp,28			; Y-Mitte
		sts	dispy,temp		; ...setzen
		ldi	temp,$ff		; schwarze Schrift
		sts	dispeor,temp		; ...einstellen
		ldiw	Z,(2*AUS)		; "AUS"
		call	disp_printrom		; ...anzeigen
;
		call	rahmen			; Rahmen zeichnen
; -------------
		ret


; ----------------------------------------------
; Messung durchführen und Digramm aufbauen:

Z2:		ldi	temp,0			; DG auf Verstärkung
		sts	dgstate,temp		; ...einstellen
; -------------
		rcall	do_window		; Butterfly Tabelle aufbauen	[ 172,41 µs,   5517 cyc]
		rcall	do_fft			; FFT durchführen		[2119,66 µs,  67829 cyc]
		rcall	do_power		; erzeuge Betragsvektor		[1203,38 µs,  38508 cyc]
		rcall	skaliere		; Skaliere 16Bit --> 8Bit	[  57,34 µs,   1835 cyc]
		rcall	filter			; lösche unnötige FFT-Werte	[  24,53 µs,    785 cyc]
		rcall	sinad			; Berechne SINAD-Wert		[  31,84 µs,   1019 cyc]
;
		rcall	grafik			; Diagramm aufbauen		[1563,78 µs,  50041 cyc]
		rcall	messwert		; Messwert eintragen		[ 236,06 µs,   7554 cyc]
		rcall	bargraf			; SINAD-Bar erzeugen		[  56,31 µs,   1802 cyc]
; -------------
		ret


; ----------------------------------------------
; Menüauswahl:

Z3:		ldi	temp,1			; DG auf Menüauswahl
		sts	dgstate,temp		; ...einstellen
; -------------
		call	disp_font2		; Normalfont verwenden
		clr	temp			; Positioniere
		sts	dispx,temp		; Textanfang links
		sts	dispy,temp		; ...oben
		ldiw	Z,(2*MENUTXT)		; Menü-Überschrift
		ldi	counter,-1		; ...ohne Begrenzung
		call	disp_printrom		; ...anzeigen
;
		ldi	buffer,3		; Anzahl der Menüpunkte
		lds	const,bbsel		; ausgewählte Bandbreite
		clr	index			; erster Eintrag
Z3_LOOP:	ldiw	Z,(2*KEINEAUSWAHL)	; Kreis ohne Auswahlpunkt
		cp	index,const		; Auswahl gefunden?
		brne	z3_line			; nein...
		ldiw	Z,(2*AUSWAHLOK)		; Kreis mit Auswahlpunkt
Z3_LINE:	ldi	counter,3		; 3 Zeichen am
		call	disp_printrom		; ...Zeilenanfang ausgeben
		ldiw	Z,(2*BANDBREITE)	; zeige auf Tabellenanfang
		ldi	temp,26			; Zeichen/Eintrag
		mul	temp,index		; rechne Offset aus
		add	ZL,akku0		; ...und zeige auf
		adc	ZH,akku1		; ...nächsten
		adiw	Z,14			; ...Menüeintrag
		ldi	counter,11		; String begrenzt
		call	disp_printrom		; ...ausgeben
		call	disp_crlf		; neue Zeile anfangen
		inc	index			; nächsten Eintrag setzen
		dec	buffer			; alle Menüpunkte ausgegeben?
		brne	z3_loop			; nein...
;
		call	disp_font1		; kleine Schrift auswählen
		clr	temp			; positionieree
		sts	dispx,temp		; ...linken Rand
		ldi	temp,58			; ...unterste
		sts	dispy,temp		; ...Zeile
		ldiw	Z,(2*INFO)		; Bedienhinweise
		ldi	counter,-1		; ...unbegrenzt
		call	disp_printrom		; ...anzeigen

; Nach DG-Kurz: Filter einstellen und Menuende

		sbis	flags2,TASTEKURZ	; Kurzeingabe vorhanden?
		rjmp	z3_end			; nein...
		cbi	flags2,TASTEKURZ	; Eingabe löschen
		sts	bbsel,const		; Filterauswahl speichern
		mov	temp,const		; Aliasing-Filter
		rcall	set_filter		; ...auf Auswahl einstellen
		ldi	temp,2			; Displayanzeige
		sts	state,temp		; ...aufrufen
; -------------
Z3_END:		ret


; ********************************************************************************************
; Ende:

.include	"data.asm"
.include	"subroutines.asm"
.include	"disp.asm"
.include	"interrupt.asm"

